//+------------------------------------------------------------------+
//|                                               Sinewave S/R  .mq4 |
//|                                                                  |
//| Sinewave                                                         |
//|                                                                  |
//| Algorithm taken from book                                        |
//|     "Cybernetics Analysis for Stock and Futures"                 |
//| by John F. Ehlers                                                |
//|                                                                  |
//|                                              contact@mqlsoft.com |
//|                                          http://www.mqlsoft.com/ |
//| - adapted by Gerald Binon                 17/05/2009             |
//| - changed by Igorad, www.forex-tsd.com    19/05/2009             |
//| - text added by Scottstone                05/03/2010             |
//| - modified to use R-MESA Cut-off Frequencies,                    |
//|   by Camaron1@gmx.com                     04/02/2011             |
//+------------------------------------------------------------------+
#property copyright "Coded by Witold Wozniak"
#property link      "www.mqlsoft.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 Red
#property indicator_color2 Blue
#property indicator_color3 Yellow
#property indicator_color4 Yellow
//#property indicator_color5 Red
//#property indicator_color6 White

#property indicator_minimum -1
#property indicator_maximum 1
#property indicator_level1 0
//#property indicator_level2 0.8
//#property indicator_level3 0.8


double Sine[];
double LeadSine[];
double Cycle[];
double Smooth[];
//double sBreak[];
//double rBreak[];

double Resistance[];
double Support[];
double trend[];
extern color lineColor = White;
extern double Alpha = 0.05;
extern bool   DrawPriceTrendLines = true;
extern int Pips = 5;
extern color Col = White;
extern double BreakInPips = 6;
extern int degree=150;                 // Degree of autocorrelation for MESA (Maximum Entropy Spectrum Analysis)
extern int length=400;                 // Number of previous Bars to perform MESA calculation
extern double max_period=140;          // Upper bound for SATL cutoff period (1/frequency)
extern double min_period=20;           // Lower bound for FATL cutoff period
extern double satl_min_period=60.0;    // Lower bound for SATL (upper bound for FATL is 80% of "*")
extern datetime initial_time=D'1970.01.01 00:00'; // if backwardBars=0, get it from initial_time
extern int filterPersistenceBars=100;  // The SATL and FATL cutoff periods are re-calculated every "*" bars
extern int backwardBars=0;             // The indicator is drawn backward "*" bars       ** Check if == 0 ** (MESA_Cutoff_frequencies problems??
int buffers = 0;
int drawBegin = 0;
int flag;
int flag2,flag3;
string name = "";
string PB = "";
string End = "";
string Text[];
double tempReal, rad2Deg, deg2Rad, label;
int    ilow, ihigh;
datetime sTime, rTime;



int init() {
    drawBegin = 8;
    initBuffer(Sine, "Sine", DRAW_LINE);
    initBuffer(LeadSine, "Lead", DRAW_LINE);
    initBuffer(Cycle);
    initBuffer(Smooth);
    initBuffer(Resistance); // "ResisT",DRAW_ARROW, STYLE_DASHDOT,1);
    initBuffer(Support);
    initBuffer(trend);
    //initBuffer(sBreak);
    //initBuffer(rBreak);
    
    //int flag = NULL;
    
    IndicatorBuffers(buffers);
    IndicatorShortName("Sinewave_MESA with S&R[" + DoubleToStr(Alpha, 2) + "]");      
    tempReal = MathArctan(1.0);
    rad2Deg = 45.0 / tempReal;
    deg2Rad = 1.0 / rad2Deg;
    return (0);
    
    /*SetIndexBuffer(3,sBreak);
	 SetIndexBuffer(4,rBreak);	
	 SetIndexStyle(3,DRAW_ARROW,0,1);
	 SetIndexStyle(4,DRAW_ARROW,0,1);	
	 SetIndexArrow(3, 159);
	 SetIndexArrow(4, 159);*/
}


 int deinit()
  {
   for(int i = ObjectsTotal() - 1; i >= 0; i--)
     {
       string label = ObjectName(i);      
       string DrawText = ObjectName(i); 
       ObjectDelete(DrawText);
       if(StringSubstr(label, 0, 18) != "Support Resistance")
       
           continue;
       ObjectDelete(label);
       ObjectDelete(DrawText);
          
     }
   return(0);
  }  
  
  
int start() {
    if (Bars <= drawBegin) return (0);
    int countedBars = IndicatorCounted(), currents;
    if (countedBars < 0) return (-1);
    if (countedBars > 0) countedBars--;
    int s, limit = Bars - countedBars - 1;
    currents=s;

    
    for (s = limit; s >= 0; s--) {
        Smooth[s] = (P(s) + 2.0 * P(s + 1) + 2.0 * P(s + 2) + P(s + 3)) / 6.0;
        Cycle[s] = (1.0 - 0.5 * Alpha) * (1.0 - 0.5 * Alpha) * (Smooth[s] - 2.0 * Smooth[s + 1] + Smooth[s + 2]) 
            + 2.0 * (1.0 - Alpha) * Cycle[s + 1]
            - (1.0 - Alpha) * (1.0 - Alpha) * Cycle[s + 2];    
        double period = iCustom(NULL,0, "R-MESA_Cutoff_frequencies",degree,length,max_period,min_period,satl_min_period,backwardBars,0,1);
        int DCPeriod = MathFloor(period);
        double RealPart = 0.0;
        double ImagPart = 0.0;
        for (int count = 0; count < DCPeriod; count++) {
            RealPart += MathSin(deg2Rad * 360.0 * count / DCPeriod) * Cycle[s + count];
            ImagPart += MathCos(deg2Rad * 360.0 * count / DCPeriod) * Cycle[s + count];
        }
        double DCPhase;
        if (MathAbs(ImagPart) > 0.0000001) {
            DCPhase = rad2Deg * MathArctan(RealPart / ImagPart);
        } else if (MathAbs(ImagPart) <= 0.0000001) {
    	      if (RealPart >= 0.0) {
    	          DCPhase = 90.0;
    	      } else { 
    	          DCPhase = -90.0;
    	      }
        }
        DCPhase += 90.0;
        if (ImagPart < 0) {
            DCPhase += 180.0;
        }
        if (DCPhase > 315.0) {
            DCPhase -= 360.0;
        }
        Sine[s] = MathSin(DCPhase * deg2Rad);
        LeadSine[s] = MathSin((DCPhase + 45.0) * deg2Rad);        
        if (DCPhase == 180 && LeadSine[s + 1] > 0) {
            LeadSine[s] = MathSin(45 * deg2Rad);
        }
        if (DCPhase == 0 && LeadSine[s + 1] < 0) {
            LeadSine[s] = MathSin(225 * deg2Rad);
        }
 //-------------------------------------------------------------------------------------//       
        
        //Attemp to draw a resistance support line at cross over
       
        //if (DrawPriceTrendLines == true)
        //{
        
        trend[s] = trend[s+1];
        
        if (LeadSine[s] > Sine[s] && LeadSine[s+1] < Sine[s+1]) trend[s] = 1;
        if (LeadSine[s] < Sine[s] && LeadSine[s+1] > Sine[s+1]) trend[s] =-1;
        //------------------Support------------------------------------------
        //Comment("Flag 1 = ",flag,"\n","Flag 2 = ",flag2,"\n","Flag 3 = ",flag3);
                
        if(trend[s]>0)
        {
            
            
            Support[s] = Support[s+1];
               if (trend[s] != trend[s+1])
               {
               Support[s] = Low[s];
               sTime = Time[s];
               }
           DrawPriceTrendLines(sTime, Time[s], Support[s], Support[s], Crimson, STYLE_SOLID);
           
           if(Support[s]>Resistance[s+1] && trend[s] != trend[s+1] && flag!=-1 && flag2!=1)
           {                                                                                   //--Pull Back
           name = "sPB"; DrawText(sTime, Low[s], name, Col);  
           flag2 =1; 
           flag =1;
          }
          
          if(Support[s]<Resistance[s+1] && trend[s] != trend[s+1] && flag==1 && flag2==-1)
          {                                                                                    //--Empty
           name = "S"; DrawText(sTime, Low[s], name, Col);  
          }
        
          if(flag2==-1 && trend[s] != trend[s+1] && flag!=1)
          {                                                                      //----------------End
          name = "sEnd";DrawText(sTime, Low[s], name, Col); 
          flag = 1;flag2 =-1; flag3 = -1;
          }     
            
          /*double sB = Support[s] - BreakInPips; //Alert(sB);
            if(Close[s]< sB && flag == -1 && Support[s]<Resistance[s+1] && trend[s]>0)
            {           
            sBreak[s]= sB;
            rBreak[s]=NULL;
            flag = 1; 
            }*/    
        } 
            
         //----------------Resistance-----------------------------------------
         if(trend[s]<0)
           {
             
            Resistance[s] = Resistance[s+1];
               if (trend[s] != trend[s+1])
               {
               Resistance[s] = High[s];
               rTime = Time[s];
               }
            double rPrice =  Resistance[s+1] + (Pips * Point);
              
            DrawPriceTrendLines(rTime, Time[s],Resistance[s], Resistance[s], White,STYLE_SOLID);
               
            if(Resistance[s]<Support[s+1] && trend[s] != trend[s+1] && flag!=1 && flag2!=-1)
           {                                                                                        //---PullBack
            name = "rPB"; DrawText(rTime, High[s], name, Col);  
            flag2 =-1; 
            flag = -1;
           }
           
           if(Resistance[s]>Support[s+1] && trend[s] != trend[s+1] && flag==-1 && flag2 ==1)
           {                                                                                       //---Empty
            name = "R"; DrawText(rTime, High[s], name, Col);  
           }    
          
           if(flag2==1 && trend[s] != trend[s+1] && flag!=-1)
           {                                                                    //----------------End
           name = "rEnd"; DrawText(rTime, High[s], name, Col); 
           flag=-1;flag2 =1; flag3= 1;
           }          
                       
          }
          /*double rB = Resistance[s] + BreakInPips; 
            if(Close[s]>rB && flag == 1 && Resistance[s]>Support[s+1] && trend[s]<0)
            {
            rBreak[s]=rB;
            sBreak[s]=NULL;
            flag = -1;
            }*/           
         
        }         

     
    return (0);
}

double P(int index) {
    return ((High[index] + Low[index]) / 2.0);
}

void initBuffer(double array[], string label = "", int type = DRAW_NONE, int arrow = 0, int style = EMPTY, int width = EMPTY, color clr = CLR_NONE) {
    SetIndexBuffer(buffers, array);
    SetIndexLabel(buffers, label);
    SetIndexEmptyValue(buffers, EMPTY_VALUE);
    SetIndexDrawBegin(buffers, drawBegin);
    SetIndexShift(buffers, 0);
    SetIndexStyle(buffers, type, style, width);
    SetIndexArrow(buffers, arrow);
    buffers++;
}


void DrawPriceTrendLines(datetime x1, datetime x2, double y1, double y2, color lineColor, double style) 
                                                             
  {
   string label = "Support Resistance" + DoubleToStr(x1, 0);
   ObjectDelete(label);
   ObjectCreate(label, OBJ_TREND, 0, x1, y1, x2, y2, 0, 0);
   ObjectSet(label, OBJPROP_RAY, 0);
   ObjectSet(label, OBJPROP_COLOR, lineColor);
   ObjectSet(label, OBJPROP_STYLE, style);
   ObjectSet(label, OBJPROP_WIDTH, 3);
  }
  
  
void DrawText(datetime x1, double y1, color lineColor, double style)

{
string DrawText1 = name + DoubleToStr(x1, 0);
ObjectDelete("End");
ObjectDelete("rPB");
ObjectDelete("sPB");
ObjectCreate(DrawText1, OBJ_TEXT, 0, x1, y1, 0, 0);
ObjectSetText(DrawText1, name, 8, "Comic Sans MS", Col);
}

/*void DrawDot(datetime x1, double y1, color lineColor, double style) 
                                                             
  {
   string  Dot = "BreakOut" + DoubleToStr(x1, 0);
   ObjectDelete(Dot);
   ObjectCreate(Dot, OBJ_TREND, 0, x1, y1, 0, 0);
   ObjectSet(Dot, OBJPROP_ARROWCODE, 0);
   ObjectSet(Dot, OBJPROP_COLOR, lineColor);
   ObjectSet(Dot, OBJPROP_STYLE, style);
   ObjectSet(Dot, OBJPROP_WIDTH, 3);
  }*/